#ifndef VL_H_
#define VL_H_

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/* error difinitions */
#define VL_NO_ERROR 0
#define VL_SOCKET_ERROR 1
#define VL_ID_DUPLICATED 2
#define VL_TOO_MANY_SOCKET 3

/* state definition */
#define VL_NOT_EXIST 0
#define VL_LISTENING 1
#define VL_CONNECTED 2
#define VL_JUST_CREATED 3
#define VL_SENDING 4
#define VL_RECIVING 5

/* mouse event definition */
#define VL_EVENT_MOUSEMOVE 0
#define VL_EVENT_LBUTTONDOWN 1
#define VL_EVENT_RBUTTONDOWN 2
#define VL_EVENT_MBUTTONDOWN 3
#define VL_EVENT_LBUTTONUP 4
#define VL_EVENT_RBUTTONUP 5
#define VL_EVENT_MBUTTONUP 6
#define VL_EVENT_LBUTTONDBLCLK 7
#define VL_EVENT_RBUTTONDBLCLK 8
#define VL_EVENT_MBUTTONDBLCLK 9
#define VL_EVENT_FLAG_LBUTTON 1
#define VL_EVENT_FLAG_RBUTTON 2
#define VL_EVENT_FLAG_MBUTTON 4
#define VL_EVENT_FLAG_CTRLKEY 8
#define VL_EVENT_FLAG_SHIFTKEY 16
#define VL_EVENT_FLAG_ALTKEY 32

/* other definitions */
#define MAX_SOCKET_NUMBER 10
#define DX 640
#define DY 480
#define DEBUG

typedef struct sSockId
{
	unsigned int sock, socke;
	int id;
	int state;
	void (*mouseCallBackFunc)(int, int, int, int, void*);
	void* mouseCallBackParam;
} sockId;

sockId socks[MAX_SOCKET_NUMBER];
int numOfSockets = 0;

/* search a socket from id */
int searchSocket(int id)
{
	int i;
	for (i = 0; i < numOfSockets; i++)
	{
		if (socks[i].id == id)
		{
			return i;
		}
	}
	return -1;
}

int vlNumberedWindow(int id)
{
	unsigned int sockAppo;
	sockaddr_in laddr, raddr, laddre;
	uint_16 rlen;
	unsigned error;
   	unsigned int opt_value1;
   	char opt_value2;
   	unsigned int opt_length1 = sizeof(unsigned int), opt_length2 = sizeof(char);
	
	if (searchSocket(id) != -1)
	{
		return VL_ID_DUPLICATED;
	}
	if (numOfSockets == MAX_SOCKET_NUMBER)
	{
		return VL_TOO_MANY_SOCKET;
	}
	
	/* server services port */
   	laddr.sin_family      = AF_INET;
   	laddr.sin_port        = id;   /* server listen port */
   	laddr.sin_addr.s_addr = INADDR_ANY;    /* every IP addr can connect to the camera */
   	/* server services port */
   	laddre.sin_family      = AF_INET;
   	laddre.sin_port        = id;   /* server listen port */
   	laddre.sin_addr.s_addr = INADDR_ANY;    /* every IP addr can connect to the camera */
	
	/* create a new TCP/IP socket */
	socks[numOfSockets].sock = socket_stream();
	socks[numOfSockets].socke = socket_stream();
	
	if ((socks[numOfSockets].sock == VCRT_SOCKET_ERROR) || (socks[numOfSockets].socke == VCRT_SOCKET_ERROR)) 
    {
		print("Create stream socket failed\n");
		return VL_SOCKET_ERROR;
    } 
	
	/* Set socket option: set send buffer size for optimal speed */
	opt_value1 = 307228;
	if (setsockopt(socks[numOfSockets].sock, SOL_TCP, OPT_TBSIZE, &opt_value1, opt_length1) != VCRT_OK)
	{
		print("Could not set socket option: OPT_TBSIZE!\n");
	}
	/* Set socket option: set non-wait on recv*/
	opt_value2 = 1;
	if (setsockopt(socks[numOfSockets].sock, SOL_TCP, OPT_RECEIVE_NOWAIT, &opt_value2, opt_length2) != VCRT_OK)
	{
		print("Could not set socket option: OPT_TBSIZE!\n");
	}
	/* Set socket option: set non-wait on recv*/
	opt_value2 = 1;
	if (setsockopt(socks[numOfSockets].socke, SOL_TCP, OPT_RECEIVE_NOWAIT, &opt_value2, opt_length2) != VCRT_OK)
	{
		print("Could not set socket option: OPT_TBSIZE!\n");
	}

   	if (error = bind(socks[numOfSockets].sock, &laddr, sizeof(laddr)) != VCRT_OK) 
    {
    	print("Stream bind failed - 0x%x\n", error);
    	return VL_SOCKET_ERROR;
    }
    
    #ifdef DEBUG
		printf("Waiting for a connection\n");
	#endif
    
    if (listen(socks[numOfSockets].sock, 0) != VCRT_OK) 
    {
    	print("Listen failed\n");
    	return VL_SOCKET_ERROR;
    }
    
    /* wait for a connection */
    VCRT_selectset(&socks[numOfSockets].sock, 1, 0);

    rlen = sizeof(raddr);

    sockAppo = accept(socks[numOfSockets].sock, &raddr, &rlen);
    shutdown(socks[numOfSockets].sock, FLAG_ABORT_CONNECTION);
    socks[numOfSockets].sock = sockAppo;

    if (socks[numOfSockets].sock == VCRT_SOCKET_ERROR)
    {
		print("Accept failed\n");
	    return VL_SOCKET_ERROR;
    }
    else
    {
    	#ifdef DEBUG
    		printf("Connection accepted\n");
    		print("Connection from %d.%d.%d.%d, port %d\n",
	       (raddr.sin_addr.s_addr >> 24) & 0xFF,
	       (raddr.sin_addr.s_addr >> 16) & 0xFF,
	       (raddr.sin_addr.s_addr >>  8) & 0xFF,
	        raddr.sin_addr.s_addr        & 0xFF,
	        raddr.sin_port);
    	#endif
    }
    
    /* socket for events */
    if (error = bind(socks[numOfSockets].socke, &laddre, sizeof(laddre)) != VCRT_OK) 
    {
    	print("Stream bind failed - 0x%x\n", error);
    	return VL_SOCKET_ERROR;
    }
    
    #ifdef DEBUG
		printf("Waiting for a connection (events)\n");
	#endif
    
    if (listen(socks[numOfSockets].socke, 0) != VCRT_OK) 
    {
    	print("Listen failed\n");
    	return VL_SOCKET_ERROR;
    }
    
    /* wait for a connection */
    VCRT_selectset(&socks[numOfSockets].socke, 1, 0);

    rlen = sizeof(raddr);

    sockAppo = accept(socks[numOfSockets].socke, &raddr, &rlen);
    shutdown(socks[numOfSockets].socke, FLAG_ABORT_CONNECTION);
    socks[numOfSockets].socke = sockAppo;

    if (socks[numOfSockets].socke == VCRT_SOCKET_ERROR)
    {
		print("Accept failed\n");
	    return VL_SOCKET_ERROR;
    }
    else
    {
    	#ifdef DEBUG
    		printf("Connection accepted\n");
    		print("Connection from %d.%d.%d.%d, port %d\n",
	       (raddr.sin_addr.s_addr >> 24) & 0xFF,
	       (raddr.sin_addr.s_addr >> 16) & 0xFF,
	       (raddr.sin_addr.s_addr >>  8) & 0xFF,
	        raddr.sin_addr.s_addr        & 0xFF,
	        raddr.sin_port);
    	#endif
    }
    
    socks[numOfSockets].id = id;
    socks[numOfSockets].state = VL_LISTENING;
    socks[numOfSockets].mouseCallBackFunc = NULL;
    socks[numOfSockets].mouseCallBackParam = NULL;
    
    numOfSockets++;
    
	return VL_NO_ERROR;
}

int vlShowImage(int id)
{
	int result;
	//printf("Sending image\n");
	*((unsigned char *)ScrGetLogPage - 1) = 0;
	result=send(socks[searchSocket(id)].sock, (char *)ScrGetLogPage - 1, DX * DY + 1, 0);
	
	if (result==VCRT_ERROR) 
	{
		print("Send VCRT_ERROR=%lx\n",VCRT_geterror(socks[searchSocket(id)].sock));
		return VL_SOCKET_ERROR;
	}
	else
	{
		if (result != DX * DY + 1)
		{
	    	print("Send error wrong length=%d result=%d\n", DX * DY, result);
		}
	}
	//printf("Image sent\n");
	return VL_NO_ERROR;
}

int vlDestroyWindows(int id)
{
	if (shutdown(socks[searchSocket(id)].sock, FLAG_CLOSE_TX) != VCRT_OK)
	{
		printf("Error while closing connection\n");
		return VL_SOCKET_ERROR;
	}
	return VL_NO_ERROR;
}

void vlSetMouseCallback(int id, void (*mouseCallBackFunc)(int, int, int, int, void*), void* param)
{
	socks[searchSocket(id)].mouseCallBackFunc = mouseCallBackFunc;
	socks[searchSocket(id)].mouseCallBackParam = param;
}


void searchForEvents()
{
	int i;
	int byteRecived;
	int buff[4];
	
	for (i = 0; i < numOfSockets; i++)
	{
		byteRecived = recv(socks[i].socke, (char*)buff, 4 * sizeof(int), 0);
		if (byteRecived != 0)
		{
			socks[i].mouseCallBackFunc(buff[1], buff[2], buff[3], buff[4], socks[i].mouseCallBackParam);
		}
		//printf("Cercato\n");
	}
}

int vlWaitKey(int id, int delay)
{
	int buff[5];
	int byteRecived;
	//printf("Wait\n");
	
	buff[0] = 0;
	(*(buff)) = (char)1;
	//printf("%d\n", (char)buff[0]);
	buff[1] = delay;
	send(socks[searchSocket(id)].sock, (char*)buff, 2 * sizeof(int), 0);
	//printf("Sent\n");
	do {
		byteRecived = recv(socks[searchSocket(id)].sock, (char*)buff, 5 * sizeof(int), 0);
		//searchForEvents();
	}
	while (byteRecived == 0 || buff[0] != 1);
	//printf("Fine ciclo\n");

	/*if (byteRecived == 0)
	{
		printf("Error in WaitKey: %d\n", byteRecived);
		return 0;
	}*/
	//printf("%d %d %d %d %d\n", (char)*(buff + 3), (char)*(buff + 4), (char)*(buff + 5), (char)*(buff + 6), (char)*(buff + 7));
	return buff[1];
}

void drawRect(image* i, int x1, int y1, int x2, int y2, int color)
{
	int x, y;
	
	if (x1 > x2)
	{
		x = x1;
		x1 = x2;
		x2 = x;
	}
	if (y1 > y2)
	{
		y = y1;
		y1 = y2;
		y2 = y;
	}
	
	for (x = 0; x1 + x <= x2; x++)
	{
		ImageSetPixel(i, x1 + x, y1, color);
	}
	for (x = 0; x1 + x <= x2; x++)
	{
		ImageSetPixel(i, x1 + x, y2, color);
	}
	for (y = 0; y1 + y <= y2; y++)
	{
		ImageSetPixel(i, x1, y1 + y, color);
	}
	for (y = 0; y1 + y <= y2; y++)
	{
		ImageSetPixel(i, x2, y1 + y, color);
	}
}

void calcHist(image* i, U32* h, int x1, int y1, int x2, int y2)
{
	int x, y;
	
	for (x = 0; x < 256; x++)
		h[x] = 0;
	
	for (x = x1; x <= x2; x++)
	{
		for (y = y1; y <= y2; y++)
		{
			h[(*((U8 *)((long)((i)->st+(x)+(y)*(i)->pitch))))]++;
		}
	}
}

float compHist(U32* h1, U32 *h2)
{
	int tot = 0, c = 0, sum = 0;
	
	for (; c < 256; c++)
	{
		tot += (h1[c] - h2[c]) * (h1[c] - h2[c]);
		sum += h1[c] + h2[c];
	}
	
	return - tot / sum;
}

#endif /*VL_H_*/
